package at.jku.fim.datalinksimulation.layer1.wire;

import java.util.ArrayList;
import java.util.Iterator;

import at.jku.fim.datalinksimulation.layer1.data.RawData;

/**
 * <p>Title: OSI Simulation Framework</p>
 * This class implements a typical standard wire, connecting 2
 * devices. Not more than 2 WireConnectors can be connected. Data sent to the wire is
 * forwarded to the other connector. If data is sent directly to the wire, the
 * data is forwarded to both connected sides.
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: StandardWire.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class StandardWire extends Wire {

    /** Registered WireConnector instances  */
    protected ArrayList connectors = new ArrayList();

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.layer1.wire.Wire#connect(at.jku.fim.datalinksimulation.layer1.wire.WireConnector)
	 */
    public void connect(WireConnector connector, int portNr) throws WiringException {
        if (connectors.size() >= 2) {
            throw new WiringException(
                    "Only 2 devices can be connected to that wire!");
        } else {
        	// inform connector about being plugged
            connector.bePlugged(this, portNr);
        }
        connectors.add(connector);
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.layer1.wire.Wire#disconnect(at.jku.fim.datalinksimulation.layer1.wire.WireConnector)
	 */
    public void disconnect(WireConnector connector, int portNr)
            throws WiringException {
        if (connectors.contains(connector)) {
        	// inform connector about being un-plugged
        	connector.beUnplugged(this, portNr);
            connectors.remove(connector);
        }
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.layer1.wire.Wire#send(at.jku.fim.datalinksimulation.data.RawData,
	 *      java.util.Vector)
	 */
    public void send(WireConnector sender, RawData data) {
        if (data == null)
            return;

        // inform neighboured wires except sender
        Iterator it = connectors.iterator();
        while (it.hasNext()) {
            WireConnector observer = (WireConnector) it.next();
            if (sender == null || !sender.equals(observer))
            	observer.informAboutDataOnWire(this, (RawData) data.clone());
        }
    }
}