package at.jku.fim.datalinksimulation.layer1.wire;

import at.jku.fim.datalinksimulation.genericdevices.HardwareComponent;
import at.jku.fim.datalinksimulation.layer1.data.RawData;

/**
 * <p>Title: OSI Simulation Framework</p>
 * This interface is implemented by classes which simulate any
 * kind of wire. A wire is connected to WireConnectors (objects
 * being informed about data on the wire) and data can be sent
 * on the wire.
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: Wire.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public abstract class Wire extends HardwareComponent {

    /**
     * Send data to the wire; the data should automatically be
     * forwarded to the directly connected WireConnectors
     * @param rawData The data to send
     */
    public abstract void send(WireConnector connector, RawData rawData);

    /**
     * Connect wire to device
     * @param connector Device to connect
     * @param portNr Connect wire to port [portNr] of device
     * @throws WiringException Thrown on wiring failure
     */
    public abstract void connect(WireConnector connector, int portNr)
            throws WiringException;

    /**
     * Connect wire to device; by default connected to port 0
     * @param connector Device to connect
     * @throws WiringException Thrown on wiring failure
     */
    public final void connect(WireConnector connector) throws WiringException {
    	connect(connector, 0);
    }
    
    /**
     * Disconnect wire from device
     * @param connector Device to disconnect
     * @param portNr The portNr where the cable is currently plugged
     * @throws WiringException Thrown on wiring failure
     */
    public abstract void disconnect(WireConnector connector, int portNr)
            throws WiringException;
    
    /**
     * Disconnect wire from device; by default disconnected from port 0
     * @param connector Device to disconnect
     * @throws WiringException Thrown on wiring failure
     */
    public final void disconnect(WireConnector connector) throws WiringException {
    	disconnect(connector, 0);
    }

}