package at.jku.fim.datalinksimulation.layer2.filters;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters;
import at.jku.fim.datalinksimulation.networkstack.incoming.DecodingNetworkFilter;
import at.jku.fim.datalinksimulation.utils.CRCUtils;
import at.jku.fim.datalinksimulation.utils.ConversionUtils;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: EasyL2CRCCheckFilter.java,v 1.1 2005/10/27 13:10:07 aputzinger Exp $
 */

public class EasyL2CRCCheckFilter implements DecodingNetworkFilter {

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.NetworkFilter#getFilterOSILayer()
	 */
    public int[] getFilterOSILayer() {
        int[] result = new int[1];
        result[0] = OSI_LAYER_2;
        return result;
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#decode(at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters,
	 *      at.jku.fim.datalinksimulation.data.RawData,
	 *      at.jku.fim.datalinksimulation.utils.Logger)
	 */
    public RawData decode(NetworkStackParameters params, RawData data,
            Logger logger) {
        logger.log("Checking CRC of frame ...");
        if (!CRCUtils.validateCRC(ConversionUtils.createBitSet(data.getRawDataBulk()), logger)) {
            logger.log("CRC does not match! Frame blocked!");
            return null;
        }
        logger.log("CRC check successful.");

        return data;
    }

   /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#getDescription()
	 */
    public String getDescription() {
        return "Performs CRC check";
    }
}