package at.jku.fim.datalinksimulation.layer2.filters;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.layer2.protocols.EasyL2FrameData;
import at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters;
import at.jku.fim.datalinksimulation.networkstack.incoming.DecodingNetworkFilter;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: EasyL2SanityCheckFilter.java,v 1.1 2005/10/27 13:10:07 aputzinger Exp $
 */

public class EasyL2SanityCheckFilter implements DecodingNetworkFilter {

    /*
     *  (non-Javadoc)
     * @see at.jku.fim.datalinksimulation.networkstack.NetworkFilter#getFilterOSILayer()
     */
    public int[] getFilterOSILayer() {
        int[] result = new int[1];
        result[0] = OSI_LAYER_2;
        return result;
    }

    /*
     *  (non-Javadoc)
     * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#decode(at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters, at.jku.fim.datalinksimulation.data.RawData, at.jku.fim.datalinksimulation.utils.Logger)
     */
    public RawData decode(NetworkStackParameters params, RawData data,
            Logger logger) {
        Object obj = params.getParameter(2, "easyl2content");
        if (obj == null) {
            logger.log("No EasyL2 information found; Blocking frame!");
            return null;
        }
        EasyL2FrameData l2content = (EasyL2FrameData) obj;
        if (l2content.getPreamble() != 0x7F) {
            logger
                    .log("Preamble does not contain magic number 0x7F; Insane frame blocked!");
            return null;
        }

        return data;
    }

   /*
    *  (non-Javadoc)
    * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#getDescription()
    */
    public String getDescription() {
        return "Performs some sanity checks on received data";
    }
}