package at.jku.fim.datalinksimulation.layer7.enduser;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.layer7.Application;
import at.jku.fim.datalinksimulation.networkstack.NetworkStack;
import at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters;
import at.jku.fim.datalinksimulation.scenario.gui.ReceiverPanel;
import at.jku.fim.datalinksimulation.utils.ConversionUtils;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: TextReceivingApplication.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class TextReceivingApplication extends Application {

    /**
     * Identifier of the station this application is running on
     */
    protected int stationId;

    /**
     * Associated GUI element
     */
    protected ReceiverPanel gui = null;

    /**
     * Associated logger component
     */
    protected Logger logger;    

    /**
     * Constructor
     * @param gui Associated GUI
     * @param stationId Station ID of host
     */
    public TextReceivingApplication(ReceiverPanel gui, int stationId) {
    	super(null);
        this.stationId = stationId;
        logger = new Logger("Text Receiving Application on station "
                + stationId);
        setGui(gui);
    }

    /**
     * Constructor
     * @param stationId Station ID of host
     */
    public TextReceivingApplication(int stationId) {
        this(null, stationId);
    }

    /**
     * This method is called whenever the incoming network stack sends data to the application
     * @param params NetworkStackParameters
     * @param data RawData
     */
    public void informAboutNewData(NetworkStackParameters params, RawData data) {
        String senderStationID = "N/A";
        if (params.getParameter(NetworkStackParameters.GLOBAL_PARAMETER,
                NetworkStack.NETWORK_PARAM_SENDER_ADDRESS) != null) {
            senderStationID = params.getParameter(
                    NetworkStackParameters.GLOBAL_PARAMETER,
                    NetworkStack.NETWORK_PARAM_SENDER_ADDRESS).toString();
        }
        logger.log("Received text from station " + senderStationID);
        logger.log("Plaintext: <" + data.toString() + ">");
        logger.log("Hex: <" + ConversionUtils.stringToHex(data.toString()) + ">");

        StringBuffer sb = new StringBuffer();
        
        int[] seq = data.getRawDataBulk();
        for (int i = 0; i < seq.length; i++) {
            sb.append((char)seq[i]);
        }
        gui.getPlainText().setText(sb.toString());
        gui.setSenderID(senderStationID);
    }

    /**
     * @return Returns associated GUI
     */
    public ReceiverPanel getGui() {
        return gui;
    }

    /**
     * Set Associated GUI
     * @param gui Associated GUI
     */
    public void setGui(ReceiverPanel gui) {
        this.gui = gui;
        if (gui != null) {
            gui.setStationID("" + stationId);
        }
    }

	/**
	 * @param parent Parent logger
	 */
	public void setParentLogger(Logger parent) {
		parent.addChildLogger(logger);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.layer7.Application#getLogger()
	 */
	public Logger getLogger() {
		return logger;
	}
    
}