package at.jku.fim.datalinksimulation.scenario.Bully;

import at.jku.fim.datalinksimulation.genericdevices.NetworkStation;
import at.jku.fim.datalinksimulation.layer1.devices.Hub;
import at.jku.fim.datalinksimulation.layer1.wire.StandardWire;
import at.jku.fim.datalinksimulation.layer1.wire.Wire;
import at.jku.fim.datalinksimulation.layer1.wire.WiringException;
import at.jku.fim.datalinksimulation.layer1.wire.damagedwire.BreakableWire;
import at.jku.fim.datalinksimulation.layer7.Bully.BullyApplication;

/**
 * The scenario for the Bully task: 4 stations, 2 hubs and a breakable wire
 * @author Michael Sonntag
 * @version $Id$
 */
public class BullyScenarioModel {

    /** The one and only scenario */
    private static BullyScenarioModel systemInstance = null;

    /** Retrieve the scenario instance 
     * @return scenario instance
     */
    public static BullyScenarioModel getInstance() {
        if(systemInstance==null) {
            systemInstance=new BullyScenarioModel();
        }
        return systemInstance;
    }

    // The Bully applications
    private BullyApplication station0App=new BullyApplication(0,4);
    private BullyApplication station1App=new BullyApplication(1,4);
    private BullyApplication station2App=new BullyApplication(2,4);
    private BullyApplication station3App=new BullyApplication(3,4);

    // Their wires
    private Wire wire0 = null;
    private Wire wire1 = null;
    private Wire wire2 = null;
    private Wire wire3 = null;

    // The wire between the hubs
    private BreakableWire breakableWire = null;

    // Both hubs
    private Hub hubLeft = null;
    private Hub hubRight = null;

    // Networking stations
    private NetworkStation station0 = null;
    private NetworkStation station1 = null;
    private NetworkStation station2 = null;
    private NetworkStation station3 = null;

    /** May not be instantiated outside of Singleton class
     */
    private BullyScenarioModel() {
        initScenario();
    }

    /**
     * initScenario
     */
    private void initScenario() {
        // Create instances of infrastructure
        wire0=new StandardWire();
        wire1=new StandardWire();
        wire2=new StandardWire();
        wire3=new StandardWire();
        breakableWire = new BreakableWire();
        hubLeft=new Hub("Hub 1",3);
        hubRight=new Hub("Hub 2",3);
        station0=new NetworkStation(new Integer(0));
        station1=new NetworkStation(new Integer(1));
        station2=new NetworkStation(new Integer(2));
        station3=new NetworkStation(new Integer(3));
        // Connect infrastructure
        try {
            // connect station 0 with left hub
            wire0.connect(station0);
            wire0.connect(hubLeft,hubLeft.getNextFreePort());
            // connect station 3 with left hub
            wire2.connect(station2);
            wire2.connect(hubLeft,hubLeft.getNextFreePort());
            // connect station 1 with right hub
            wire1.connect(station1);
            wire1.connect(hubRight,hubRight.getNextFreePort());
            // connect station 2 with right hub
            wire3.connect(station3);
            wire3.connect(hubRight,hubRight.getNextFreePort());
            // connect both hubs using DAMAGED(!) wire
            breakableWire.connect(hubLeft,hubLeft.getNextFreePort());
            breakableWire.connect(hubRight,hubRight.getNextFreePort());
        } catch (WiringException ex) {
            ex.printStackTrace();
        }

    }
    
    /** Start the scenario: The applications start to run */
    public void startScenario() {
        // start applications on the stations
        station0.runApplication(station0App);
        station1.runApplication(station1App);
        station2.runApplication(station2App);
        station3.runApplication(station3App);
    }

    /** Retrieve the breakable wire
     * @return the breakable wire
     */
    public BreakableWire getBreakableWire() {
        return breakableWire;
    }

    /** Retrieve the application of station 0
     * @return the application
     */
    public BullyApplication getStation0App() {
        return station0App;
    }

    /** Retrieve the application of station 1
     * @return the application
     */
    public BullyApplication getStation1App() {
        return station1App;
    }

    /** Retrieve the application of station 2
     * @return the application
     */
    public BullyApplication getStation2App() {
        return station2App;
    }

    /** Retrieve the application of station 3
     * @return the application
     */
    public BullyApplication getStation3App() {
        return station3App;
    }

}
