package at.jku.fim.datalinksimulation.scenario.Bully.gui;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.TitledBorder;

import at.jku.fim.datalinksimulation.scenario.Bully.BullyScenarioModel;
import at.jku.fim.datalinksimulation.scenario.gui.LogPanel;

/**
 * <p>The Bully simulation. Creates and arranges all the GUI elements.</p>
 * <p>Copyright: (c) 2004</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Michael Sonntag
 * @version $Id$
 */

public class BullySimulationFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private JPanel mainPanel = new JPanel();

    private JPanel titleAndSimulationPanel = new JPanel();

    private JPanel informationPanel = new JPanel();

    private BullyPanel station0Panel=new BullyPanel();
    private BullyPanel station1Panel=new BullyPanel();
    private BullyPanel station2Panel=new BullyPanel();
    private BullyPanel station3Panel=new BullyPanel();
    private JToggleButton partitionButton = new JToggleButton();

    private JPanel backgroundPanel=new JPanel();
    private Icon icon=new ImageIcon(getClass().getResource("BullyInfrastructure.jpg"));
    private JLabel backgroundLabel = new JLabel();

    private LogPanel logPanel=new LogPanel();
    private TitledBorder logBorder=new TitledBorder("");

    private JPanel titlePanel=new JPanel();
    private JLabel titleLabel=new JLabel();
    private TitledBorder titleBorder=new TitledBorder("");

    /** Create a new Bully simluation */
    public BullySimulationFrame() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** Update the log windows */
    public void logUpdate() {
        logPanel.showSelectedLog();
    }

    /** Reset all stations, starts the scenario and updates the log. May only be called once! */
    public void stationsReset() {
        station0Panel.setApp(BullyScenarioModel.getInstance().getStation0App());
        station1Panel.setApp(BullyScenarioModel.getInstance().getStation1App());
        station2Panel.setApp(BullyScenarioModel.getInstance().getStation2App());
        station3Panel.setApp(BullyScenarioModel.getInstance().getStation3App());
        BullyScenarioModel.getInstance().startScenario();
        logPanel.updateLogs();
    }

    private void jbInit() throws Exception {
        setTitle("Bully Simulation");
        getContentPane().setLayout(new BorderLayout());
        mainPanel.setLayout(new BorderLayout());
        titleAndSimulationPanel.setLayout(new BorderLayout());
        informationPanel.setLayout(new BorderLayout());

        titlePanel.setBackground(Color.orange);
        titlePanel.setLayout(new BorderLayout());
        titleLabel.setFont(new java.awt.Font("Dialog", 1, 17));
        titleLabel.setBorder(titleBorder);
        titleLabel.setText("Bully Simulation");
        titleLabel.setHorizontalAlignment(JLabel.CENTER);

        backgroundLabel.setBorder(BorderFactory.createEtchedBorder());
        backgroundLabel.setIcon(icon);
        backgroundLabel.setBounds(new Rectangle(0,0,805,283));

        station0Panel.setBounds(new Rectangle(20,30,225,82));
        station1Panel.setBounds(new Rectangle(543,30,225,82));
        station2Panel.setBounds(new Rectangle(20,153,225,82));
        station3Panel.setBounds(new Rectangle(543,153,225, 82));
        
        partitionButton.setBackground(Color.green);
        partitionButton.setSelected(false);
        partitionButton.setText("OK");
        partitionButton.setBounds(new Rectangle(345,170,70,23));
        partitionButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e) {
                partitionButtonToggled();
            }            
        });                

        backgroundPanel.setLayout(null);
        backgroundPanel.setMinimumSize(new Dimension(800,280));
        backgroundPanel.setPreferredSize(new Dimension(800,280));
        backgroundPanel.setMaximumSize(new Dimension(800,280));
        backgroundPanel.add(station0Panel,null);
        backgroundPanel.add(station1Panel,null);
        backgroundPanel.add(station2Panel,null);
        backgroundPanel.add(station3Panel,null);
        backgroundPanel.add(partitionButton,null);
        backgroundPanel.add(backgroundLabel,null);
        
        titlePanel.add(titleLabel, BorderLayout.CENTER);
        titleAndSimulationPanel.add(titlePanel, BorderLayout.NORTH);
        titleAndSimulationPanel.add(backgroundPanel, BorderLayout.CENTER);
        mainPanel.add(titleAndSimulationPanel, BorderLayout.NORTH);

        logPanel.setBorder(logBorder);
        informationPanel.add(logPanel, BorderLayout.CENTER);
        mainPanel.add(informationPanel, BorderLayout.CENTER);

        getContentPane().add(mainPanel, BorderLayout.CENTER);
        
        BullyScenarioModel.getInstance().getStation0App().setGui(station0Panel);
        BullyScenarioModel.getInstance().getStation1App().setGui(station1Panel);
        BullyScenarioModel.getInstance().getStation2App().setGui(station2Panel);
        BullyScenarioModel.getInstance().getStation3App().setGui(station3Panel);

        stationsReset();
    }

    /** Overridden so we can exit when window is closed
     * @param e the window event
     */
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    private void partitionButtonToggled() {
        Color damagingActive = Color.RED;
        Color damagingInactive = Color.GREEN;
        partitionButton.setSelected(partitionButton.isSelected());
        if (partitionButton.isSelected()) {
            partitionButton.setBackground(damagingActive);
            partitionButton.setText("Broken");
            BullyScenarioModel.getInstance().getBreakableWire().setBroken(true);
        } else {
            partitionButton.setBackground(damagingInactive);
            partitionButton.setText("OK");
            BullyScenarioModel.getInstance().getBreakableWire().setBroken(false);
        }
    }

}