package at.jku.fim.datalinksimulation.scenario;

import at.jku.fim.datalinksimulation.genericdevices.InterlinkingDevice;
import at.jku.fim.datalinksimulation.genericdevices.NetworkStation;
import at.jku.fim.datalinksimulation.layer1.devices.Hub;
import at.jku.fim.datalinksimulation.layer1.wire.StandardWire;
import at.jku.fim.datalinksimulation.layer1.wire.Wire;
import at.jku.fim.datalinksimulation.layer1.wire.WiringException;
import at.jku.fim.datalinksimulation.layer1.wire.damagedwire.DamagedWire;
import at.jku.fim.datalinksimulation.layer1.wire.damagedwire.SimpleDamagingStrategy;
import at.jku.fim.datalinksimulation.layer2.devices.CRCGuard;
import at.jku.fim.datalinksimulation.layer7.enduser.TextReceivingApplication;
import at.jku.fim.datalinksimulation.layer7.enduser.TextSendingApplication;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: SystemScenarioModel.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class SystemScenarioModel {

    public static SystemScenarioModel systemInstance = null;

    public static SystemScenarioModel getInstance() {
        if (systemInstance == null)
            systemInstance = new SystemScenarioModel();
        return systemInstance;
    }

    // instance things start here
    
    // declare cables
    private DamagedWire damagedWire;
    private Wire hub2hubWire;
    private Wire hub2iHubWire;
    private Wire station10Wire;
    private Wire station11Wire;
    
    // declare interlinking devices
    private InterlinkingDevice hub1;
    private InterlinkingDevice hub2;
    private InterlinkingDevice iHub;
    
    protected NetworkStation station1 = null;
    protected NetworkStation station10 = null;
    protected NetworkStation station11 = null;
    protected TextSendingApplication sendAppStation1 = null;
    protected TextReceivingApplication recAppStation10 = null;
    protected TextReceivingApplication recAppStation11 = null;

    // may not be instantiated outside of Singleton class
    private SystemScenarioModel() {
        initScenario();
    }

    /**
     * initScenario
     */
    private void initScenario() {
        // create instances of infrastructure
    	
    	// create cables
        damagedWire = new DamagedWire(new SimpleDamagingStrategy());
        hub2hubWire = new StandardWire();
        hub2iHubWire = new StandardWire();
        station10Wire = new StandardWire();
        station11Wire = new StandardWire();
        
        // create interlinking devices
        hub1 = new Hub("Hub 1", 4);
        hub2 = new Hub("Hub 2", 4);
        iHub = new CRCGuard("CRCGuard", 4);
        
        // create network stations
        station1 = new NetworkStation(new Integer(1));
        station10 = new NetworkStation(new Integer(10));
        station11 = new NetworkStation(new Integer(11));
        
        // connect infrastructure
        try {
            // connect sender (station 1) with hub using DAMAGED cable
            damagedWire.connect(station1);
            damagedWire.connect(hub1, hub1.getNextFreePort());
            
            // connect the 2 "normal" hubs (-> station 10)
            hub2hubWire.connect(hub1, hub1.getNextFreePort());
            hub2hubWire.connect(hub2, hub2.getNextFreePort());

            // connect the first hub with the intelligent hub  (-> station 11)
            hub2iHubWire.connect(hub1, hub1.getNextFreePort());
            hub2iHubWire.connect(iHub, iHub.getNextFreePort());
            
            // connect the second normal hub with station 10
        	station10Wire.connect(station10);
        	station10Wire.connect(hub2, hub2.getNextFreePort());
            
            // connect the iHub with station 11
        	station11Wire.connect(station11);
        	station11Wire.connect(iHub, iHub.getNextFreePort());
        } catch (WiringException ex) {
            ex.printStackTrace();
        }

        // start applications on the stations
        sendAppStation1 = new TextSendingApplication(1);
        recAppStation10 = new TextReceivingApplication(10);
        recAppStation11 = new TextReceivingApplication(11);
        station1.runApplication(sendAppStation1);
        station10.runApplication(recAppStation10);
        station11.runApplication(recAppStation11);
    }

	public DamagedWire getDamagedWire() {
		return damagedWire;
	}

	public InterlinkingDevice getHub1() {
		return hub1;
	}

	public InterlinkingDevice getHub2() {
		return hub2;
	}

	public Wire getHub2hubWire() {
		return hub2hubWire;
	}

	public Wire getHub2iHubWire() {
		return hub2iHubWire;
	}

	public InterlinkingDevice getIHub() {
		return iHub;
	}

	public TextReceivingApplication getRecAppStation10() {
		return recAppStation10;
	}

	public TextReceivingApplication getRecAppStation11() {
		return recAppStation11;
	}

	public TextSendingApplication getSendAppStation1() {
		return sendAppStation1;
	}

	public NetworkStation getStation1() {
		return station1;
	}

	public NetworkStation getStation10() {
		return station10;
	}

	public Wire getStation10Wire() {
		return station10Wire;
	}

	public NetworkStation getStation11() {
		return station11;
	}

	public Wire getStation11Wire() {
		return station11Wire;
	}    
}

